@@ -47,6 +47,7 @@ import {
     AST_BlockStatement,
     AST_Class,
     AST_ClassExpression,
+    AST_ClassStaticBlock,
     AST_DefaultAssign,
     AST_DefClass,
     AST_Definitions,
@@ -152,6 +153,11 @@ AST_Scope.DEFMETHOD("drop_unused", function(compressor) {
             });
         }
         if (node === self) return;
+        if (node instanceof AST_Class) {
+            if (node.has_side_effects(compressor)) {
+                node.visit_nondeferred_class_parts(tw);
+            }
+        }
         if (node instanceof AST_Defun || node instanceof AST_DefClass) {
             var node_def = node.name.definition();
             const in_export = tw.parent() instanceof AST_Export;
@@ -160,23 +166,7 @@ AST_Scope.DEFMETHOD("drop_unused", function(compressor) {
                     in_use_ids.set(node_def.id, node_def);
                 }
             }
-            if (node instanceof AST_DefClass) {
-                if (
-                    node.extends
-                    && (node.extends.has_side_effects(compressor)
-                    || node.extends.may_throw(compressor))
-                ) {
-                    node.extends.walk(tw);
-                }
-                for (const prop of node.properties) {
-                    if (
-                        prop.has_side_effects(compressor) ||
-                        prop.may_throw(compressor)
-                    ) {
-                        prop.walk(tw);
-                    }
-                }
-            }
+
             map_add(initializations, node_def.id, node);
             return true; // don't go in nested scopes
         }
@@ -241,9 +231,9 @@ AST_Scope.DEFMETHOD("drop_unused", function(compressor) {
                         if (!in_use || fixed_ids.has(def.id) && fixed_ids.get(def.id) !== node) {
                             return maintain_this_binding(parent, node, node.right.transform(tt));
                         }
-                    } else if (!in_use) return in_list ? MAP.skip : make_node(AST_Number, node, {
-                        value: 0
-                    });
+                    } else if (!in_use) {
+                        return in_list ? MAP.skip : make_node(AST_Number, node, { value: 0 });
+                    }
                 }
             }
             if (scope !== self) return;
@@ -283,14 +273,24 @@ AST_Scope.DEFMETHOD("drop_unused", function(compressor) {
                     }
                 }
             }
-            if ((node instanceof AST_Defun || node instanceof AST_DefClass) && node !== self) {
+            if (node instanceof AST_DefClass && node !== self) {
+                const def = node.name.definition();
+                descend(node, this);
+                const keep_class = def.global && !drop_funcs || in_use_ids.has(def.id);
+                if (!keep_class) {
+                    const kept = node.drop_side_effect_free(compressor);
+                    if (kept == null) {
+                        def.eliminated++;
+                        return in_list ? MAP.skip : make_node(AST_EmptyStatement, node);
+                    }
+                    return kept;
+                }
+                return node;
+            }
+            if (node instanceof AST_Defun && node !== self) {
                 const def = node.name.definition();
                 const keep = def.global && !drop_funcs || in_use_ids.has(def.id);
-                // Class "extends" and static blocks may have side effects
-                const has_side_effects = !keep
-                    && node instanceof AST_Class
-                    && node.has_side_effects(compressor);
-                if (!keep && !has_side_effects) {
+                if (!keep) {
                     def.eliminated++;
                     return in_list ? MAP.skip : make_node(AST_EmptyStatement, node);
                 }
@@ -386,9 +386,7 @@ AST_Scope.DEFMETHOD("drop_unused", function(compressor) {
                   case 1:
                     return body[0];
                   default:
-                    return in_list ? MAP.splice(body) : make_node(AST_BlockStatement, node, {
-                        body: body
-                    });
+                    return in_list ? MAP.splice(body) : make_node(AST_BlockStatement, node, { body });
                 }
             }
             // certain combination of unused name + side effect leads to:
@@ -431,7 +429,7 @@ AST_Scope.DEFMETHOD("drop_unused", function(compressor) {
                 }
                 return node;
             }
-            if (node instanceof AST_Scope) {
+            if (node instanceof AST_Scope && !(node instanceof AST_ClassStaticBlock)) {
                 const save_scope = scope;
                 scope = node;
                 descend(node, this);
@@ -470,7 +468,11 @@ AST_Scope.DEFMETHOD("drop_unused", function(compressor) {
             }
             return true;
         }
-        if (node instanceof AST_Scope) {
+        if (node instanceof AST_Class) {
+            descend();
+            return true;
+        }
+        if (node instanceof AST_Scope && !(node instanceof AST_ClassStaticBlock)) {
             var save_scope = scope;
             scope = node;
             descend();
