@@ -74,7 +74,9 @@ import {
     AST_ConciseMethod,
     AST_PrivateGetter,
     AST_PrivateMethod,
+    AST_SymbolPrivateProperty,
     AST_PrivateSetter,
+    AST_PrivateIn,
     AST_Conditional,
     AST_Const,
     AST_Constant,
@@ -141,6 +143,7 @@ import {
     AST_Throw,
     AST_Toplevel,
     AST_Try,
+    AST_TryBlock,
     AST_Unary,
     AST_UnaryPostfix,
     AST_UnaryPrefix,
@@ -255,7 +258,7 @@ function OutputStream(options) {
     if (options.shorthand === undefined)
         options.shorthand = options.ecma > 5;
 
-    // Convert comment option to RegExp if neccessary and set up comments filter
+    // Convert comment option to RegExp if necessary and set up comments filter
     var comment_filter = return_false; // Default case, throw all comments away
     if (options.comments) {
         let comments = options.comments;
@@ -395,10 +398,12 @@ function OutputStream(options) {
         mappings.forEach(function(mapping) {
             try {
                 let { name, token } = mapping;
-                if (token.type == "name" || token.type === "privatename") {
-                    name = token.value;
-                } else if (name instanceof AST_Symbol) {
-                    name = token.type === "string" ? token.value : name.name;
+                if (name !== false) {
+                    if (token.type == "name" || token.type === "privatename") {
+                        name = token.value;
+                    } else if (name instanceof AST_Symbol) {
+                        name = token.type === "string" ? token.value : name.name;
+                    }
                 }
                 options.source_map.add(
                     mapping.token.file,
@@ -659,14 +664,17 @@ function OutputStream(options) {
         if (!start) return;
         var printed_comments = self.printed_comments;
 
-        // There cannot be a newline between return and its value.
-        const return_with_value = node instanceof AST_Exit && node.value;
+        // There cannot be a newline between return/yield and its value.
+        const keyword_with_value = 
+            node instanceof AST_Exit && node.value
+            || (node instanceof AST_Await || node instanceof AST_Yield)
+                && node.expression;
 
         if (
             start.comments_before
             && printed_comments.has(start.comments_before)
         ) {
-            if (return_with_value) {
+            if (keyword_with_value) {
                 start.comments_before = [];
             } else {
                 return;
@@ -679,10 +687,12 @@ function OutputStream(options) {
         }
         printed_comments.add(comments);
 
-        if (return_with_value) {
+        if (keyword_with_value) {
             var tw = new TreeWalker(function(node) {
                 var parent = tw.parent();
                 if (parent instanceof AST_Exit
+                    || parent instanceof AST_Await
+                    || parent instanceof AST_Yield
                     || parent instanceof AST_Binary && parent.left === node
                     || parent.TYPE == "Call" && parent.expression === node
                     || parent instanceof AST_Conditional && parent.condition === node
@@ -701,7 +711,7 @@ function OutputStream(options) {
                 }
             });
             tw.push(node);
-            node.value.walk(tw);
+            keyword_with_value.walk(tw);
         }
 
         if (current_pos == 0) {
@@ -1077,6 +1087,12 @@ function OutputStream(options) {
             return true;
     });
 
+    PARENS(AST_Chain, function(output) {
+        var p = output.parent();
+        if (!(p instanceof AST_Call || p instanceof AST_PropAccess)) return false;
+        return p.expression === this;
+    });
+
     PARENS(AST_PropAccess, function(output) {
         var p = output.parent();
         if (p instanceof AST_New && p.expression === this) {
@@ -1223,7 +1239,7 @@ function OutputStream(options) {
     }
 
     AST_StatementWithBody.DEFMETHOD("_do_print_body", function(output) {
-        force_statement(this.body, output);
+        print_maybe_braced_body(this.body, output);
     });
 
     DEFPRINT(AST_Statement, function(self, output) {
@@ -1552,7 +1568,7 @@ function OutputStream(options) {
                 b = b.body;
             } else break;
         }
-        force_statement(self.body, output);
+        print_maybe_braced_body(self.body, output);
     }
     DEFPRINT(AST_If, function(self, output) {
         output.print("if");
@@ -1569,7 +1585,7 @@ function OutputStream(options) {
             if (self.alternative instanceof AST_If)
                 self.alternative.print(output);
             else
-                force_statement(self.alternative, output);
+                print_maybe_braced_body(self.alternative, output);
         } else {
             self._do_print_body(output);
         }
@@ -1618,7 +1634,7 @@ function OutputStream(options) {
     DEFPRINT(AST_Try, function(self, output) {
         output.print("try");
         output.space();
-        print_braced(self, output);
+        self.body.print(output);
         if (self.bcatch) {
             output.space();
             self.bcatch.print(output);
@@ -1628,6 +1644,9 @@ function OutputStream(options) {
             self.bfinally.print(output);
         }
     });
+    DEFPRINT(AST_TryBlock, function(self, output) {
+        print_braced(self, output);
+    });
     DEFPRINT(AST_Catch, function(self, output) {
         output.print("catch");
         if (self.argname) {
@@ -1679,7 +1698,9 @@ function OutputStream(options) {
             output.space();
         }
         if (self.imported_names) {
-            if (self.imported_names.length === 1 && self.imported_names[0].foreign_name.name === "*") {
+            if (self.imported_names.length === 1 &&
+                self.imported_names[0].foreign_name.name === "*" &&
+                !self.imported_names[0].foreign_name.quote) {
                 self.imported_names[0].print(output);
             } else {
                 output.print("{");
@@ -1713,14 +1734,31 @@ function OutputStream(options) {
     DEFPRINT(AST_NameMapping, function(self, output) {
         var is_import = output.parent() instanceof AST_Import;
         var definition = self.name.definition();
+        var foreign_name = self.foreign_name;
         var names_are_different =
             (definition && definition.mangled_name || self.name.name) !==
-            self.foreign_name.name;
+            foreign_name.name;
+        if (!names_are_different &&
+            foreign_name.name === "*" &&
+            foreign_name.quote != self.name.quote) {
+                // export * as "*"
+            names_are_different = true;
+        }
+        var foreign_name_is_name = foreign_name.quote == null;
         if (names_are_different) {
             if (is_import) {
-                output.print(self.foreign_name.name);
+                if (foreign_name_is_name) {
+                    output.print(foreign_name.name);
+                } else {
+                    output.print_string(foreign_name.name, foreign_name.quote);
+                }
             } else {
-                self.name.print(output);
+                if (self.name.quote == null) {
+                    self.name.print(output);
+                } else {
+                    output.print_string(self.name.name, self.name.quote);
+                }
+                
             }
             output.space();
             output.print("as");
@@ -1728,10 +1766,18 @@ function OutputStream(options) {
             if (is_import) {
                 self.name.print(output);
             } else {
-                output.print(self.foreign_name.name);
+                if (foreign_name_is_name) {
+                    output.print(foreign_name.name);
+                } else {
+                    output.print_string(foreign_name.name, foreign_name.quote);
+                }
             }
         } else {
-            self.name.print(output);
+            if (self.name.quote == null) {
+                self.name.print(output);
+            } else {
+                output.print_string(self.name.name, self.name.quote);
+            }
         }
     });
 
@@ -1743,8 +1789,10 @@ function OutputStream(options) {
             output.space();
         }
         if (self.exported_names) {
-            if (self.exported_names.length === 1 && self.exported_names[0].name.name === "*") {
-                self.exported_names[0].print(output);
+            if (self.exported_names.length === 1 &&
+                self.exported_names[0].name.name === "*" &&
+                !self.exported_names[0].name.quote) {
+                    self.exported_names[0].print(output);
             } else {
                 output.print("{");
                 self.exported_names.forEach(function(name_export, i) {
@@ -1795,7 +1843,10 @@ function OutputStream(options) {
                 if (node instanceof AST_Scope && !(node instanceof AST_Arrow)) {
                     return true;
                 }
-                if (node instanceof AST_Binary && node.operator == "in") {
+                if (
+                    node instanceof AST_Binary && node.operator == "in"
+                    || node instanceof AST_PrivateIn
+                ) {
                     return walk_abort;  // makes walk() return true
                 }
             });
@@ -1868,7 +1919,7 @@ function OutputStream(options) {
             ? output.option("ie8")
             : !is_identifier_string(
                 prop,
-                output.option("ecma") >= 2015 || output.option("safari10")
+                output.option("ecma") >= 2015 && !output.option("safari10")
             );
 
         if (self.optional) output.print("?.");
@@ -1912,6 +1963,10 @@ function OutputStream(options) {
     });
     DEFPRINT(AST_UnaryPrefix, function(self, output) {
         var op = self.operator;
+        if (op === "--" && output.last().endsWith("!")) {
+            // avoid printing "<!--"
+            output.print(" ");
+        }
         output.print(op);
         if (/^[a-z]/i.test(op)
             || (/[+-]$/.test(op)
@@ -1929,8 +1984,7 @@ function OutputStream(options) {
         var op = self.operator;
         self.left.print(output);
         if (op[0] == ">" /* ">>" ">>>" ">" ">=" */
-            && self.left instanceof AST_UnaryPostfix
-            && self.left.operator == "--") {
+            && output.last().endsWith("--")) {
             // space is mandatory to avoid outputting -->
             output.print(" ");
         } else {
@@ -1938,17 +1992,7 @@ function OutputStream(options) {
             output.space();
         }
         output.print(op);
-        if ((op == "<" || op == "<<")
-            && self.right instanceof AST_UnaryPrefix
-            && self.right.operator == "!"
-            && self.right.expression instanceof AST_UnaryPrefix
-            && self.right.expression.operator == "--") {
-            // space is mandatory to avoid outputting <!--
-            output.print(" ");
-        } else {
-            // the space is optional depending on "beautify"
-            output.space();
-        }
+        output.space();
         self.right.print(output);
     });
     DEFPRINT(AST_Conditional, function(self, output) {
@@ -2036,15 +2080,19 @@ function OutputStream(options) {
         output.print("new.target");
     });
 
+    /** Prints a prop name. Returns whether it can be used as a shorthand. */
     function print_property_name(key, quote, output) {
         if (output.option("quote_keys")) {
-            return output.print_string(key);
+            output.print_string(key);
+            return false;
         }
         if ("" + +key == key && key >= 0) {
             if (output.option("keep_numbers")) {
-                return output.print(key);
+                output.print(key);
+                return false;
             }
-            return output.print(make_num(key));
+            output.print(make_num(key));
+            return false;
         }
         var print_string = ALL_RESERVED_WORDS.has(key)
             ? output.option("ie8")
@@ -2054,9 +2102,11 @@ function OutputStream(options) {
                     : !is_identifier_string(key, true)
             );
         if (print_string || (quote && output.option("keep_quoted_props"))) {
-            return output.print_string(key, quote);
+            output.print_string(key, quote);
+            return false;
         }
-        return output.print_name(key);
+        output.print_name(key);
+        return true;
     }
 
     DEFPRINT(AST_ObjectKeyVal, function(self, output) {
@@ -2065,28 +2115,29 @@ function OutputStream(options) {
             return def ? def.mangled_name || def.name : self.name;
         }
 
-        var allowShortHand = output.option("shorthand");
-        if (allowShortHand &&
-            self.value instanceof AST_Symbol &&
-            is_identifier_string(
-                self.key,
-                output.option("ecma") >= 2015 || output.option("safari10")
-            ) &&
-            get_name(self.value) === self.key &&
-            !ALL_RESERVED_WORDS.has(self.key)
+        const try_shorthand = output.option("shorthand") && !(self.key instanceof AST_Node);
+        if (
+            try_shorthand
+            && self.value instanceof AST_Symbol
+            && get_name(self.value) === self.key
+            && !ALL_RESERVED_WORDS.has(self.key)
         ) {
-            print_property_name(self.key, self.quote, output);
-
-        } else if (allowShortHand &&
-            self.value instanceof AST_DefaultAssign &&
-            self.value.left instanceof AST_Symbol &&
-            is_identifier_string(
-                self.key,
-                output.option("ecma") >= 2015 || output.option("safari10")
-            ) &&
-            get_name(self.value.left) === self.key
+            const was_shorthand = print_property_name(self.key, self.quote, output);
+            if (!was_shorthand) {
+                output.colon();
+                self.value.print(output);
+            }
+        } else if (
+            try_shorthand
+            && self.value instanceof AST_DefaultAssign
+            && self.value.left instanceof AST_Symbol
+            && get_name(self.value.left) === self.key
         ) {
-            print_property_name(self.key, self.quote, output);
+            const was_shorthand = print_property_name(self.key, self.quote, output);
+            if (!was_shorthand) {
+                output.colon();
+                self.value.left.print(output);
+            }
             output.space();
             output.print("=");
             output.space();
@@ -2154,6 +2205,7 @@ function OutputStream(options) {
         if (self.key instanceof AST_SymbolMethod) {
             if (is_private) output.print("#");
             print_property_name(self.key.name, self.quote, output);
+            self.key.add_source_map(output);
         } else {
             output.with_square(function() {
                 self.key.print(output);
@@ -2184,6 +2236,16 @@ function OutputStream(options) {
         }
         self._print_getter_setter(type, true, output);
     });
+    DEFPRINT(AST_PrivateIn, function(self, output) {
+        self.key.print(output);
+        output.space();
+        output.print("in");
+        output.space();
+        self.value.print(output);
+    });
+    DEFPRINT(AST_SymbolPrivateProperty, function(self, output) {
+        output.print("#" + self.name);
+    });
     DEFPRINT(AST_ConciseMethod, function(self, output) {
         var type;
         if (self.is_generator && self.async) {
@@ -2232,12 +2294,18 @@ function OutputStream(options) {
     });
 
     const r_slash_script = /(<\s*\/\s*script)/i;
+    const r_starts_with_script = /^\s*script/i;
     const slash_script_replace = (_, $1) => $1.replace("/", "\\/");
     DEFPRINT(AST_RegExp, function(self, output) {
         let { source, flags } = self.getValue();
         source = regexp_source_fix(source);
         flags = flags ? sort_regexp_flags(flags) : "";
+
+        // Avoid outputting end of script tag
         source = source.replace(r_slash_script, slash_script_replace);
+        if (r_starts_with_script.test(source) && output.last().endsWith("<")) {
+            output.print(" ");
+        }
 
         output.print(output.to_utf8(`/${source}/${flags}`, false, true));
 
@@ -2251,12 +2319,15 @@ function OutputStream(options) {
         }
     });
 
-    function force_statement(stat, output) {
+    /** if, for, while, may or may not have braces surrounding its body */
+    function print_maybe_braced_body(stat, output) {
         if (output.option("braces")) {
             make_block(stat, output);
         } else {
             if (!stat || stat instanceof AST_EmptyStatement)
                 output.force_semicolon();
+            else if (stat instanceof AST_Let || stat instanceof AST_Const || stat instanceof AST_Class)
+                make_block(stat, output);
             else
                 stat.print(output);
         }
@@ -2358,8 +2429,22 @@ function OutputStream(options) {
         AST_ObjectSetter,
         AST_PrivateGetter,
         AST_PrivateSetter,
+        AST_ConciseMethod,
+        AST_PrivateMethod,
     ], function(output) {
-        output.add_mapping(this.key.end, this.key.name);
+        output.add_mapping(this.start, false /*name handled below*/);
+    });
+
+    DEFMAP([
+        AST_SymbolMethod,
+        AST_SymbolPrivateProperty
+    ], function(output) {
+        const tok_type = this.end && this.end.type;
+        if (tok_type === "name" || tok_type === "privatename") {
+            output.add_mapping(this.end, this.name);
+        } else {
+            output.add_mapping(this.end);
+        }
     });
 
     DEFMAP([ AST_ObjectProperty ], function(output) {
