@@ -16,12 +16,16 @@ import * as sandbox from "./sandbox.js";
 import { minify } from "../main.js";
 import { defaults } from "../lib/utils/index.js";
 
+const minify_catch_error = async (...args) =>
+    minify(...args).catch(error => ({ error }))
+
 const __filename = fileURLToPath(import.meta.url);
 const __dirname = path.dirname(__filename);
 
 var MAX_GENERATED_TOPLEVELS_PER_RUN = 1;
 var MAX_GENERATION_RECURSION_DEPTH = 12;
 var INTERVAL_COUNT = 100;
+var RANDOM_SEED = Date.now();
 
 var STMT_ARG_TO_ID = Object.create(null);
 var STMTS_TO_USE = [];
@@ -80,6 +84,12 @@ for (var i = 2; i < process.argv.length; ++i) {
         STMT_SECOND_LEVEL_OVERRIDE = STMT_ARG_TO_ID[name];
         if (!(STMT_SECOND_LEVEL_OVERRIDE >= 0)) throw new Error('Unknown statement name; use -? to get a list');
         break;
+      case '--seed':
+        RANDOM_SEED = Number(process.argv[++i]);
+        if (isNaN(RANDOM_SEED)) {
+            throw new Error('invalid random seed ' + process.argv[i])
+        }
+        break;
       case '--no-catch-redef':
         catch_redef = false;
         break;
@@ -110,6 +120,7 @@ for (var i = 2; i < process.argv.length; ++i) {
         println('<number>: generate this many cases (if used must be first arg)');
         println('-v: print every generated test case');
         println('-V: print every 100th generated test case');
+        println('--seed: initialize the random seed, for determinism');
         println('-t <int>: generate this many toplevels per run (more take longer)');
         println('-r <int>: maximum recursion depth for generator (higher takes longer)');
         println('-s1 <statement name>: force the first level statement to be this one (see list below)');
@@ -314,11 +325,20 @@ var funcs = 0;
 var called = Object.create(null);
 var labels = 10000;
 
-function rng(max) {
-    var r = randomBytes(2).readUInt16LE(0) / 65536;
-    return Math.floor(max * r);
+// LCG deterministic random number generator.
+// https://stackoverflow.com/a/72732727/1011311 (adapted)
+function makeRng(seed) {
+    var m = 2**35 - 31
+    var a = 185852
+    var s = seed % m
+    return function (max) {
+        const rand_float = ((s = s * a % m) / m);
+        return Math.floor(max * rand_float);
+    }
 }
 
+const rng = makeRng(RANDOM_SEED);
+
 function strictMode() {
     return use_strict && rng(4) == 0 ? '"use strict";' : '';
 }
@@ -971,7 +991,7 @@ function errorln(msg) {
 }
 
 async function try_beautify(code, result, printfn) {
-    var beautified = await minify(code, {
+    var beautified = await minify_catch_error(code, {
         compress: false,
         mangle: false,
         output: {
@@ -1007,7 +1027,7 @@ async function log_suspects(minify_options, component) {
             var o = JSON.parse(JSON.stringify(options));
             o[name] = flip;
             m[component] = o;
-            var result = await minify(original_code, m);
+            var result = await minify_catch_error(original_code, m);
             if (result.error) {
                 errorln("Error testing options." + component + "." + name);
                 errorln(result.error.stack);
@@ -1032,7 +1052,7 @@ async function log_suspects(minify_options, component) {
 async function log_rename(options) {
     var m = JSON.parse(JSON.stringify(options));
     m.rename = false;
-    var result = await minify(original_code, m);
+    var result = await minify_catch_error(original_code, m);
     if (result.error) {
         errorln("Error testing options.rename");
         errorln(result.error.stack);
@@ -1046,7 +1066,7 @@ async function log_rename(options) {
     }
 }
 
-async function log(options) {
+async function log(options, round) {
     if (!ok) errorln('\n\n\n\n\n\n!!!!!!!!!!\n\n\n');
     errorln("//=============================================================");
     if (!ok) errorln("// !!!!!! Failed... round " + round);
@@ -1097,14 +1117,16 @@ var terser_code, terser_result, ok;
 
 async function main() {
     for (var round = 1; round <= num_iterations; round++) {
-        process.stdout.write(round + " of " + num_iterations + "\r");
+        if (!process.env.CI || round % 100 === 0 || round === 1) {
+            process.stdout.write(round + " of " + num_iterations + "\r");
+        }
 
         original_code = createTopLevelCode();
         original_result = sandbox.run_code(original_code);
         const options_sets = (typeof original_result != "string" ? fallback_options : minify_options)
 
         for (const options of options_sets) {
-            terser_code = await minify(original_code, JSON.parse(options));
+            terser_code = await minify_catch_error(original_code, JSON.parse(options));
             if (!terser_code.error) {
                 terser_code = terser_code.code;
                 terser_result = sandbox.run_code(terser_code);
@@ -1115,7 +1137,7 @@ async function main() {
                     ok = terser_code.name == original_result.name;
                 }
             }
-            if (verbose || (verbose_interval && !(round % INTERVAL_COUNT)) || !ok) await log(options);
+            if (verbose || (verbose_interval && !(round % INTERVAL_COUNT)) || !ok) await log(options, round);
             else if (typeof original_result != "string") {
                 println("//=============================================================");
                 println("// original code");
